#include <SDL/SDL.h>
#include "../headers/PlayerClass.h"
#include "../headers/SpriteClass.h"
#include "../headers/SoundClass.h"
#include "../headers/gmath.h"
#include "../headers/functions.h"
#include "../headers/defines.h"

extern Sprite mysprite;
extern Sound mysound;

Player::Player()
{
	x = 0, y = 0, hspeed = 0.0, vspeed = 0.0, w = 24, h = 46, image_index = 0.0, facing = 1, sprite_index = -1, animated = 1, hanging_left = 0, hanging_right = 0, weapon = 0, attacking = 0;
	set_sprite(0);	
	current_triangle[0] = current_triangle[1] = current_triangle[2] = current_triangle[3] = -1; 
	//current weapons: 0 - sword, 1 - pistol
}

bool Player::set_sprite( unsigned char sprite )
{
	if (sprite_index == sprite + weapon || (sprite_index == SPR_PLAYER_HANG && sprite == SPR_PLAYER_HANG))
		return 0;
	
	image_index = 0; //only when changing sprite
	
	switch(sprite)
	{
		case SPR_PLAYER_STAND:
			sprite_index = sprite + weapon;
			images = 1;
			tex_w = 64;
			tex_h = 64;
			img_w = 64;
			img_h = 64;			
			image_speed = 1;
		break;
		case SPR_PLAYER_RUN:		
			sprite_index = sprite + weapon;
			images = 9;
			tex_w = 1024;
			tex_h = 64;
			img_w = 64;
			img_h = 64;
			image_speed = 2;
		break;
		case SPR_PLAYER_AT:
			sprite_index = sprite + weapon;
			images = 13 - weapon;
			tex_w = 1024;
			tex_h = 64;
			img_w = 64;
			img_h = 64;
			image_speed = 4;
		break;		
		case SPR_PLAYER_JUMP:
			sprite_index = sprite + weapon;
			images = 2;
			tex_w = 128;
			tex_h = 64;
			img_w = 64;
			img_h = 64;			
			image_speed = 4;
		break;
		case SPR_PLAYER_HANG:
			sprite_index = sprite;
			images = 10;
			tex_w = 1024;
			tex_h = 64;
			img_w = 64;
			img_h = 64;			
			image_speed = 3;
		break;
	}
	return 1;
}

void Player::set_x(int _x)
{
    x = _x;  
}

void Player::set_y(int _y)
{
    y = _y;
}

void Player::set_xy(int _x, int _y)
{
	x = _x;
    y = _y;
}

bool Player::can_move( unsigned int terrain_grid[][9], int _x, int _y )
{
	int next_triangle[4];
	for ( char i = 0; i <= 3; i++ )
		next_triangle[i] = current_triangle[i];							

	if ( place_free(terrain_grid, x - w/2, y - h/2, _x - w/2, _y - h/2, next_triangle, 0) && place_free(terrain_grid, x + w/2, y - h/2, _x + w/2, _y - h/2, next_triangle, 1) && place_free(terrain_grid, x + w/2, y + h/2, _x + w/2, _y + h/2, next_triangle, 2) && place_free(terrain_grid, x - w/2, y + h/2, _x - w/2, _y + h/2, next_triangle, 3))
	{									
		for ( char i = 0; i <= 3; i++ )
			current_triangle[i] = next_triangle[i];
			
		return 1;
	}
	
	return 0;
}     

bool Player::pl_place_free( unsigned int terrain_grid[][9], int _x, int _y )
{
	if ( place_free(terrain_grid, x - w/2, y - h/2, _x - w/2, _y - h/2, current_triangle[0]) && place_free(terrain_grid, x + w/2, y - h/2, _x + w/2, _y - h/2, current_triangle[1]) && place_free(terrain_grid, x + w/2, y + h/2, _x + w/2, _y + h/2, current_triangle[2]) && place_free(terrain_grid, x - w/2, y + h/2, _x - w/2, _y + h/2, current_triangle[3]))
		return 1;
	
	return 0;
}     

void Player::set_my_triangles( unsigned int terrain_grid[][9], unsigned int triangles )
{
	set_triangles( terrain_grid, current_triangle, x, y, w, h, triangles );
}

void Player::move(Uint8 *keystates, unsigned int terrain_grid[][9], unsigned int triangles)
{			
	if (current_triangle[0] == -1) //first time
		set_triangles( terrain_grid, current_triangle, x, y, w, h, triangles );	

	
	if (!attacking)
	{
		if(!hanging_left && !hanging_right)
		{
			if ( keystates[ KEY_RIGHT ] )
				hspeed = min(2, hspeed + .2, 4.0);
			else if ( !keystates[ KEY_LEFT ] && hspeed > 0 )
				hspeed = max(2, hspeed - .2, 0.0);	
			
			if ( keystates[ KEY_LEFT ] )
				hspeed = max(2, hspeed - .2, -4.0);
			else if ( !keystates[ KEY_RIGHT ] && hspeed < 0 )
				hspeed = min(2, hspeed + .2, 0.0);			
		}		
	}
		
	if(!hanging_left && !hanging_right)		
	{
		if (hspeed > 0)
		{	
			facing = 1;
			bool moved = 0;		
			for( char i = 0; i < hspeed; i++)
			{
				for( char j = 0; j < 4; j++ )
				{							
					if (can_move(terrain_grid, x + 1, y - j))
					{													
						set_xy(x + 1, y - j);
						moved = 1;
						break;
					}
					hspeed = max(2, 0.0, hspeed - .15); //slopes make player go slower
				}
				if (!moved)
					hspeed = 0;	
				check_hanging(terrain_grid);						
			}
			if (moved && (!place_free(terrain_grid, x + w/2, y + h/2, x + w/2, y + h/2 + 4, current_triangle[2]) || !place_free(terrain_grid, x - w/2, y + h/2, x - w/2, y + h/2 + 4, current_triangle[3])))
				if (!mysound.sound_is_playing(SND_PLAYER_FOOTSTEP))
					mysound.play_sound(SND_PLAYER_FOOTSTEP);		
		}
		else if (hspeed < 0)
		{
			facing = 0;
			bool moved = 0;		
			for( char i = 0; i > hspeed; i--)
			{
				for( char j = 0; j < 4; j++ )
				{
					if (can_move(terrain_grid, x - 1, y - j))
					{
						set_xy(x - 1, y - j);
						moved = 1;
						break;
					}
					hspeed = min(2, 0.0, hspeed + .15); //slopes make player go slower
				}
				if (!moved)
					hspeed = 0;
				check_hanging(terrain_grid);		
			}		
			if (moved && (!place_free(terrain_grid, x + w/2, y + h/2, x + w/2, y + h/2 + 4, current_triangle[2]) || !place_free(terrain_grid, x - w/2, y + h/2, x - w/2, y + h/2 + 4, current_triangle[3])))
				if (!mysound.sound_is_playing(SND_PLAYER_FOOTSTEP))
					mysound.play_sound(SND_PLAYER_FOOTSTEP);				
		}
	
	
		if (!attacking)
		{
			check_hanging(terrain_grid);	

			if (vspeed > 0)
			{
				for( char i = 0; i < vspeed; i++)
				{
					if (can_move(terrain_grid, x, y + 1))
						set_y(y + 1);
					else
					{
						mysound.sound_set_volume(SND_PLAYER_LAND, 128 / int(10 / vspeed));
						mysound.play_sound(SND_PLAYER_LAND);
						vspeed = 0;
					}
				}
			}
			else
			{	
				for( char i = 0; i > vspeed; i--)
				{
					if (can_move(terrain_grid, x, y - 1))
						set_y(y - 1);
					else
						vspeed = 0;
				}		
			}	

		//gravity
		if (pl_place_free(terrain_grid, x, y + 1))
			vspeed = min(2, vspeed + 0.4, 6.0);	
		
			vspeed = min(2, vspeed, 12.0);
		}
	}
	
	
	//jumping
	if (sprite_index == SPR_PLAYER_JUMP && (int)image_index == 3)
	{
		jump(keystates, terrain_grid);		
		animated = 0;
		image_index = 4;
	}
	
	//jumping
	if (sprite_index == SPR_PLAYER_HANG && (int)image_index == 9)
	{
		bool hr = hanging_right, hl = hanging_left;
		
		do
		{
			if (can_move(terrain_grid, x - hanging_right + hanging_left, y))
			{
				set_x(x - hanging_right + hanging_left);
				check_hanging(terrain_grid);
			}
		}
		while (get_hanging_r() || get_hanging_l());
		
		if (can_move(terrain_grid, x - 6 * hr + 6 * hl, y))
			set_x(x - 6 * hr + 6 * hl);		
		
		if (can_move(terrain_grid, x, y - 6))
		{
			set_y( y - 6 );
			vspeed = -7.0;
		}		
	}	
		
	update_sprite(terrain_grid);	
}

void Player::show(Uint8 *keystates, unsigned int terrain_grid[][9], int mouse_x, int mouse_y, int camera_x, int camera_y)
{	
	//highlighting triangles------------------------------
	
	/*    glDisable(GL_TEXTURE_2D);
	    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		glBegin( GL_TRIANGLES );
		
		glColor3f(1.0,1.0,.0);
		
		// Draw terrain triangles
		for(char i = 0; i <= 3; i++)
		{
				glVertex3f( terrain_grid[current_triangle[i]][0], terrain_grid[current_triangle[i]][1], 0 );
				glVertex3f( terrain_grid[current_triangle[i]][2], terrain_grid[current_triangle[i]][3], 0 );
	    		glVertex3f( terrain_grid[current_triangle[i]][4], terrain_grid[current_triangle[i]][5], 0 );		
		}
		glColor3f(1.0,1.0,1.0);			    		
		glEnd();				
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	
	*/
	
	//Finish attacking
	if (sprite_index == SPR_PLAYER_AT + weapon && (int)image_index == 11)
		attacking = 0;	
	
	//Increase image_index
	if (animated)
		image_index += (double)image_speed / 10.0;

	//Loop animation
	if ((image_index >= images && animated == 1) || images == 1)
		image_index = 0;	
		
	//Draw player
	//Body
	Draw_Texture(mysprite.get(sprite_index), x - 18 - w/2 + hanging_left * 4, y - h/2 + (hanging_right || hanging_left) * 4, img_w, img_h, tex_w, tex_h, (int)image_index, facing, 0);		

	if (weapon == 1 && !hanging_left && !hanging_right ) //Pistol
	{
		//Head
		bool running = (sprite_index == SPR_PLAYER_RUN + weapon);
		Draw_Texture(mysprite.get(SPR_PLAYER_HEAD), x - w/2 + 8 - !facing * 4 + (!facing * running) - (facing * running), y + 10 - h/2 - running, 16, 16, 16, 16, 0, facing, 0);		

		int angle = (int)radtodeg(point_direction((double)x,(double)y,(double)mouse_x + (double)camera_x,(double)mouse_y + (double)camera_y));
		//Arm
		if (facing) //to the right
			Draw_Texture(mysprite.get(SPR_PLAYER_ARM_PISTOL), x - w/2 + 10, y + 22 - h/2, 20, 6, 32, 32, 0, facing, angle );
		else
		{
			if (angle == 0)
				angle = 1;
			Draw_Texture(mysprite.get(SPR_PLAYER_ARM_PISTOL), x - w/2 + 18, y + 22 - h/2, 20, 6, 32, 32, 0, facing, angle);			
		}
	}

	//TESTING
//	if (/*hanging_left || hanging_right*/	/*place_free(terrain_grid, x + w/2, y + h/2, 2, current_triangle) /*&&*/ //!place_free(terrain_grid, x + w/2, y - h/2 + 1, 1, current_triangle))
/*	{
		glDisable(GL_TEXTURE_2D);
		glBegin( GL_TRIANGLES );
		
		glColor3f((double)hanging_left,(double)hanging_right,.0);
		
		// Draw triangles	
		glVertex3f( x - 50, y, 0 );
		glVertex3f( x - 60, y + 30, 0 );
   		glVertex3f( x - 40, y + 30, 0 );		

		glColor3f(1.0,1.0,1.0);			    		
		glEnd();				
	}*/
	//4 vertices
	
	/*glDisable(GL_TEXTURE_2D);	
	glBegin( GL_POINTS );
		
	glColor3f(1.0,.0,.0);
		
	glVertex2f( x - w/2 , y - h/2);
	glVertex2f( x + w/2, y - h/2 );		
	glVertex2f( x - w/2, y + h/2 );
	glVertex2f( x + w/2, y + h/2 );			

	glColor3f(1.0,1.0,1.0);			    		
	glEnd();		*/
	
}

void Player::jump(Uint8 *keystates, unsigned int terrain_grid[][9])
{
	if (!place_free(terrain_grid, x + w/2, y + h/2, x + w/2, y + h/2 + 4, current_triangle[2]) || !place_free(terrain_grid, x - w/2, y + h/2, x - w/2, y + h/2 + 4, current_triangle[3]))
	{		
		if ( keystates[ KEY_UP ] )			
			vspeed = -7;	
		else
			vspeed = -5;
		mysound.play_sound(SND_PLAYER_JUMP);					
	}	
}

void Player::check_hanging(unsigned int terrain_grid[][9])
{
	//right	
	if ( !place_free( terrain_grid, x + w/2, y - h/2, x + w/2, y - h/2 + 4, current_triangle[1] )/* || !place_free(terrain_grid, x + w/2, y - h/2 + 1, 1, current_triangle))*/ )
	{
		hanging_right = 1;
		hspeed = 0;
		animated = 0;
		facing = 1;
		
		do
		{
			if ( can_move(terrain_grid, x - 1, y))
				set_x(x - 1);
		}
		while (!place_free( terrain_grid, x + w/2, y - h/2, x + w/2, y - h/2 + 4, current_triangle[1]));
		if ( can_move(terrain_grid, x + 1, y))
			set_x(x + 1);				
	}			
	else
		hanging_right = 0;
	//left
	if ( !place_free( terrain_grid, x - w/2, y - h/2, x - w/2, y - h/2 + 4, current_triangle[0] ) )
	{
		hanging_left = 1;
		hspeed = 0;
		animated = 0;
		facing = 0;
		
		do
		{
			if ( can_move(terrain_grid, x + 1, y))
				set_x(x + 1);
		}
		while (!place_free( terrain_grid, x - w/2, y - h/2, x - w/2, y - h/2 + 4, current_triangle[0]));
		if ( can_move(terrain_grid, x - 1, y))
			set_x(x - 1);		
	}			
	else
		hanging_left = 0;
		
}

void Player::update_sprite(unsigned int terrain_grid[][9])
{
	//set sprite
	if ((hanging_left || hanging_right) /*&& vspeed == 0*/)		
		set_sprite(SPR_PLAYER_HANG);	
	else
	{	
		//on air
		if (place_free(terrain_grid, x+ w/2, y + h/2, x + w/2, y + h/2 + 4, current_triangle[2]) && place_free(terrain_grid, x - w/2, y + h/2, x - w/2, y + h/2 + 4, current_triangle[3]))
		{
			set_sprite(SPR_PLAYER_JUMP);		
			animated = 0;
			if (vspeed < 0)
				image_index = 0;
			else
				image_index = 1;
		}		
		else //not on air
		{
			if ( hspeed )
			{
				animated = 1;
				set_sprite(SPR_PLAYER_RUN);
			}
			else //not moving and on the floor
			{
				animated = 1;
				if (attacking)
					set_sprite(SPR_PLAYER_AT);						
				else
					set_sprite(SPR_PLAYER_STAND);			
			}
		}
	}
}

void Player::switch_weapons()
{
	if (!attacking)
	{
		set_sprite(sprite_index);

		if (weapon < 1)
			weapon += 1;
		else
			weapon = 0;
	}
}

void Player::set_attacking(bool atk)
{
	if (!attacking)
	{
		if (!weapon) //melee
		{
			attacking = (atk && abs(hspeed) <= 1 && sprite_index != SPR_PLAYER_JUMP + weapon);  //only melee ones
			if (attacking)
			{
				image_index = 0;
				hspeed = .0;
			}
		}
	}
}
